function PWMs = P3M(L, angles, u, phase)

    % Maximize/minimize phi between 0 and 2pi
    for i = 1:length(phase)
        while(phase(i)>=2*pi)
            phase(i) = phase(i)-2*pi;
        end
        while(phase(i)<0)
            phase(i) = phase(i)+2*pi;
        end
    end
    
    % Transform the phase/angles in integers between 0 and 15000
    phase  = floor(phase*30000/pi);
    angles = floor(angles*30000/pi);
    
    % Create the array of angles
    x = [0, angles', 30000-fliplr(angles'), 30000 , 30000+angles', 60000-fliplr(angles')];
    
    % Create the corresponding array of states
    if(L==2)
        y = [0, u, fliplr(u(1:end-1)),0, 1, fliplr(u(1:end-1)), 0, u, 0];
        offset   = 0;
    else
        y = [0, u, fliplr(u(1:end-1)),0, 0, -u, -fliplr(u(1:end-1)),0 , 0];
        offset   = (L-1)/2;
    end
    
    % Compute 1 PWM
    PWM1        = zeros(1,length(phase));
    PWM1_multi1 = zeros(L-1,length(phase));
    output   = 1;
    to_check = 2;
    
    for it=1:length(phase)
        if(phase(it) == x(to_check))
            output = to_check;
            if(to_check == length(x))
                to_check = 1;
            else
                to_check = to_check+1;
            end
        end
        PWM1(it) = y(output)+offset;
    end
    
    % Normalisation
    PWM1 = PWM1./(L-1); %max(min(PWM1-(i-1),1),0);
    
    % Create the 3 separated legs:
    tiers = floor(length(phase)/3);
    PWM1_multi2 = [PWM1(:,2*tiers+1:end) PWM1(:,1:2*tiers)];
    PWM1_multi3 = [PWM1(:,tiers+1:end)   PWM1(:,1:tiers)];
   
    % Assemble
    PWMs = [PWM1; PWM1_multi2; PWM1_multi3];

end